#include <UAW API.h>



enum TerrainID : UAW_Extension_TerrainID {
	sampleTerrain = 123, // choose any number you like
};

enum ScenarioID : UAW_Extension_ScenarioID {
	sampleScenario = 123, // choose any number you like
};

constexpr auto tileSize = 2048.f; // ~2 km – good speed/space tradeoff

enum StringID : UAW_Extension_StringID {
	extensionName,
	extensionAuthor,
	extensionUrl,
	extensionDescription,
	sampleScenarioName,
	sampleScenarioDescription,
};

// Returns a string in UAW’s current language.
UAW_Char const * UAW_CALL stringFromStringID(
	UAW_Extension const *  extension, // explained later
	UAW_Extension_StringID id
) {
	switch(id) {
		case extensionName:
			return u"Sample Extension";
		case extensionAuthor:
			return u"Someone";
		case extensionUrl:
			return u"http://example.com";
		case extensionDescription:
			return u"Demonstrates the implementation of a basic UAW extension.";
		case sampleScenarioName:
			return u"Sample Scenario";
		case sampleScenarioDescription:
			return u"Demonstrates the implementation of a basic UAW scenario.";
		default:
			return u"YOU FORGOT TO DEFINE THIS STRING!";
	}
}

UAW_Extension_Terrain * UAW_CALL loadTerrain(
	UAW_Extension *             extension,
	UAW_API_TerrainLoad *       uaw,
	UAW_Extension_TerrainID     id,
	UAW_Extension_Terrain_API * result
) {
	return nullptr; // for now, simulate failure
}

UAW_Extension_Scenario * UAW_CALL createScenario(
	UAW_Extension *              extension,
	UAW_API_ScenarioPopulate *   uaw,
	UAW_Extension_ScenarioID     id,
	UAW_Extension_Scenario_API * methods
) {
	return nullptr; // for now, simulate failure
}

extern "C" __declspec(dllexport) int UAW_CALL UAW_EXTENSION_CREATE(
	UAW_API_Extension *       uaw,
	UAW_Extension_Callbacks * result,
	void *                    legacyPleaseIgnore
) {
	// If you work object-oriented, create an instance and store its address here.
	// It will be passed back to you with every callback.
	result->implementation = nullptr;

	// If you need a chance to clean up before UAW quits, place your callback here.
	result->unload = nullptr;

	// For display in the Extensions menu (from the Strings chapter sample):
	result->string      = &stringFromStringID;
	result->name        = extensionName;
	result->author      = extensionAuthor;
	result->url         = extensionUrl;
	result->description = extensionDescription;

	// Declare sample terrain:
	UAW_TerrainSpecs terrain = { };
	terrain.id             = sampleTerrain;
	terrain.latitude       = 49.f; // origin is somewhere in the Celtic Sea
	terrain.longitude      = -10.f;
	terrain.tileSidelength = tileSize;
	terrain.load           = &loadTerrain; // pass our callback
	uaw->newTerrain(terrain);

	// Declare an empty scenario using the sample terrain:
	UAW_ScenarioSpecs scenario = { };
	scenario.id        = sampleScenario;
	scenario.terrainID = sampleTerrain;
	scenario.name      = sampleScenarioName;
	scenario.summary   = sampleScenarioDescription;
	scenario.create    = &createScenario; // pass our callback
	uaw->newScenario(scenario);

	return 1; // success (zero for failure)
}